@@ -5,14 +5,8 @@
 #' the errors (\code{stopifnot()} only prints the possibly truncated source code
 #' of the expressions).
 #'
-#' There are two ways to write R expressions in the \code{...} argument.
-#'
-#' The first way is a series of R expressions (each expression is passed as an
-#' individual argument) that return vectors of \code{TRUE}'s (if \code{FALSE} is
-#' returned anywhere, an error will show up).
-#'
-#' The second way is a single R expression wrapped in \code{{}} and passed as a
-#' single argument. This expression may contain multiple sub-expressions. A
+#' For the \code{...} argument, it should be a single R expression wrapped in
+#' \code{{}}. This expression may contain multiple sub-expressions. A
 #' sub-expression is treated as a test condition if it is wrapped in \code{()}
 #' (meaning its value will be checked to see if it is a logical vector
 #' containing any \code{FALSE} values) , otherwise it is evaluated in the normal
@@ -21,9 +15,10 @@
 #' @param fact a message for the assertions when any of them fails; treated the
 #'   same way as expressions in \code{...} if it is not a character string,
 #'   which means you do not have to provide a message to this function
-#' @param ... any number of R expressions; see Details
-#' @return Invisible \code{NULL} if all expressions returned \code{TRUE},
-#'   otherwise an error is signalled and the user-provided message is emitted.
+#' @param ... an R expression; see Details
+#' @return For \code{assert()}, invisible \code{NULL} if all expressions
+#'   returned \code{TRUE}, otherwise an error is signalled and the user-provided
+#'   message is emitted. For \code{\%==\%}, \code{TRUE} or \code{FALSE}.
 #' @note The internal implementation of \code{assert()} is different with the
 #'   \code{stopifnot()} function in R \pkg{base}: (1) the custom message
 #'   \code{fact} is emitted if an error occurs; (2) \code{assert()} requires the
@@ -36,7 +31,21 @@
 #'   an error upon the first failed assertion, and will ignore the rest of
 #'   assertions.
 #' @export
-#' @examples ## The first way to write assertions --------------------
+#' @examples
+#' ## The first way to write assertions -------------------
+#'
+#' assert('T is bad for TRUE, and so is F for FALSE', {T=FALSE;F=TRUE
+#' (T!=TRUE)  # note the parentheses
+#' (F!=FALSE)})
+#'
+#' assert('A Poisson random number is non-negative', {
+#' x = rpois(1, 10)
+#' (x >= 0)
+#' (x > -1)  # () is optional because it's the last expression
+#' })
+#'
+#'
+#' ## The second way to write assertions --------------------
 #'
 #' assert('one equals one', 1==1)
 #' assert('seq and : produce equal sequences', seq(1L, 10L) == 1L:10L)
@@ -58,20 +67,8 @@
 #'
 #' # no message
 #' assert(!FALSE, TRUE, is.na(NA))
-#'
-#'
-#' ## The second way to write assertions -------------------
-#'
-#' assert('T is bad for TRUE, and so is F for FALSE', {T=FALSE;F=TRUE
-#' (T!=TRUE)  # note the parentheses
-#' (F!=FALSE)})
-#'
-#' assert('A Poisson random number is non-negative', {
-#' x = rpois(1, 10)
-#' (x >= 0)
-#' x > -1  # do not need () here because it's the last expression
-#' })
 assert = function(fact, ...) {
+  opt = options(testit.asserting = TRUE); on.exit(options(opt), add = TRUE)
   mc = match.call()
   # match.call() uses the arg order in the func def, so fact is always 1st arg
   fact = NULL
@@ -112,11 +109,29 @@ assert2 = function(fact, exprs, envir, all = TRUE) {
 #' @description The infix operator \code{\%==\%} is simply an alias of the
 #'   \code{\link{identical}()} function to make it slightly easier and intuitive
 #'   to write test conditions. \code{x \%==\% y} is the same as
-#'   \code{identical(x, y)}.
+#'   \code{identical(x, y)}. When it is used inside \code{assert()}, a message
+#'   will be printed if the returned value is not \code{TRUE}, to show the
+#'   values of the LHS (\code{x}) and RHS (\code{y}) via \code{\link{str}()},
+#'   which can be helpful for you to check why the assertion failed.
 #' @param x,y two R objects to be compared
 #' @rdname assert
+#' @import utils
 #' @export
-`%==%` = function(x, y) identical(x, y)
+`%==%` = function(x, y) {
+  res = identical(x, y)
+  if (!res && isTRUE(getOption('testit.asserting', FALSE))) {
+    mc = match.call()
+    info = paste(capture.output({
+      cat(deparse_key(mc[[2]]), '(LHS) ==>\n')
+      str(x)
+      cat('----------\n')
+      str(y)
+      cat('<== (RHS)', deparse_key(mc[[3]]), '\n')
+    }), collapse = '\n')
+    message(info)
+  }
+  res
+}
 
 #' Run the tests of a package in its namespace
 #'
@@ -124,12 +139,14 @@ assert2 = function(fact, exprs, envir, all = TRUE) {
 #' when running tests, which allows one to use non-exported objects in the
 #' package without having to resort to the triple colon \code{\link{:::}} trick.
 #'
-#' The tests are assumed to be under the \file{testit/} directory by default,
-#' and this function also looks for the \file{tests/testit/} directory under the
-#' package installation directory when the user-provided \code{dir} does not
-#' exist. The test scripts must be named of the form \samp{test-*.R}; other R
-#' scripts will not be treated as test files (but may also be useful, e.g. you
-#' can \code{\link{source}()} them in tests).
+#' The tests are assumed to be under the \file{testit/} or \file{tests/testit/}
+#' directory by default (depending on your working directory is the package root
+#' directory or the \file{tests/} directory). This function also looks for the
+#' \file{tests/testit/} directory under the package installation directory when
+#' the user-provided \code{dir} does not exist. The test scripts must be named
+#' of the form \samp{test-*.R}; other R scripts will not be treated as test
+#' files (but may also be useful, e.g. you can \code{\link{source}()} them in
+#' tests).
 #'
 #' For \command{R CMD check}, this means the test R scripts (\file{test-*.R} are
 #' under \file{pkg_root/tests/testit/}. The R scripts are executed with
@@ -139,14 +156,14 @@ assert2 = function(fact, exprs, envir, all = TRUE) {
 #' be removed before the code is executed.
 #' @param package the package name
 #' @param dir the directory of the test files; by default, it is the directory
-#'   \file{testit/} under the current working directory
+#'   \file{testit/} or \file{tests/testit/} under the current working directory
 #' @return \code{NULL}. All test files are executed, unless an error occurs.
 #' @note All test scripts (\samp{test-*.R}) must be encoded in UTF-8 if they
 #'   contain any multibyte characters.
 #' @seealso The \pkg{testthat} package (much more sophisticated).
 #' @export
 #' @examples \dontrun{test_pkg('testit')}
-test_pkg = function(package, dir = 'testit') {
+test_pkg = function(package, dir = c('testit', 'tests/testit')) {
   library(package, character.only = TRUE)
   path = available_dir(c(dir, system.file('tests', 'testit', package = package)))
   rs = list.files(path, '^test-.+[.][rR]$', full.names = TRUE)
@@ -164,8 +181,7 @@ test_pkg = function(package, dir = 'testit') {
         s = if (!is.null(srcref <- attr(z, 'srcref'))) {
           paste0(' at ', basename(attr(srcref, 'srcfile')$filename), '#', srcref[1])
         }
-        z[n] = paste0(z[n], s)
-        cat(z, sep = '\n')
+        cat('Error from', z[1], if (n > 1) '...', s, '\n')
       }
     )
   }
@@ -176,12 +192,13 @@ test_pkg = function(package, dir = 'testit') {
 #' The two functions \code{has_warning()} and \code{has_error()} check if an
 #' expression produces warnings and errors, respectively.
 #' @param expr an R expression
+#' @param silent logical: should the report of error messages be suppressed?
 #' @return A logical value.
 #' @export
 #' @rdname has_message
 #' @examples has_warning(1+1); has_warning(1:2+1:3)
 #'
-#' has_error(2-3); has_error(1+'a')
+#' has_error(2-3); has_error(1+'a'); has_error(stop("err"), silent = TRUE)
 has_warning = function(expr) {
   warn = FALSE
   op = options(warn = -1); on.exit(options(op))
@@ -193,6 +210,6 @@ has_warning = function(expr) {
 }
 #' @export
 #' @rdname has_message
-has_error = function(expr) {
-  inherits(try(force(expr), silent = !interactive()), 'try-error')
+has_error = function(expr, silent = !interactive()) {
+  inherits(try(force(expr), silent = silent), 'try-error')
 }
